; -----------------------------------------------------------------------------
; __          ___       ____ ___
; \ \        / (_)     |___ \__ \   RetroForth for Windows
;  \ \  /\  / / _ _ __   __) | ) |  http://www.retroforth.org
;   \ \/  \/ / | | '_ \ |__ < / /
;    \  /\  /  | | | | |___) / /_
;     \/  \/   |_|_| |_|____/____|
; -----------------------------------------------------------------------------

format PE console DLL
stack 16000
entry DLLEntryPoint     ; entry point of the executable

include "macros.fasm"
include "windows-include/win32a.inc"

section ".code" code readable writeable executable
DLLEntryPoint:
	mov	eax,TRUE
ret

;---------------------------------------------------------------------
retroforth_attach:
	pusha
	invoke GetModuleHandle, 0
	mov [hinstance], eax

	invoke GetStdHandle, STD_INPUT_HANDLE
	mov [StdIn], eax
	invoke SetConsoleMode, eax, 5

	invoke GetStdHandle, STD_OUTPUT_HANDLE
	mov [StdOut], eax

	invoke SetConsoleTitle, "RetroForth"
	popa
ret
retroforth_interpreter:
	call retroforth_attach
	mov eax, dword [rf_eax]
	mov esi, dword [rf_esi]
	call interpret
	ret
retroforth_init:
	call init_retroforth
	mov dword [rf_eax], eax
	mov dword [rf_esi], esi
ret
;---------------------------------------------------------------------
code 'bye', windows_bye
	mov dword [rf_eax], eax
	mov dword [rf_esi], esi
	pop ebx
ret
code 'emit', emit
	pusha
	mov [emit_buffer], eax  ; keep it safe
	invoke WriteConsole, [StdOut], emit_buffer, 1, 0, 0
	popa
	drop
	ret
code 'key', key
	dup
	pusha
	invoke ReadConsole, [StdIn], emit_buffer, 1, written_buffer, 0
	popa
	mov eax, [emit_buffer]
	ret
code 'LoadLibraryAs', load_dll
	embed 'zt-make'
	push eax
	drop
	upop edx
	pusha
	invoke LoadLibrary, edx
	mov dword [tempX], eax
	popa
	dup
	mov eax, dword [tempX]
	dup
	pop eax
	embed 'zt-free'
	call create
	call comma
next
code 'MapFunctionAs', load_function
	embed 'zt-make'
	push eax
	drop
	upop ebx
	upop edx
	mov  edx, [edx]
	pusha
	invoke GetProcAddress,edx,ebx
	mov dword [tempX], eax
	popa
	dup
	mov eax, dword [tempX]
	dup
	pop eax
	embed 'zt-free'
	call create
	call comma
next
code 'invoke', dll_invoke
	upop ecx
	cmp ecx, 0
	jz .run
.chain:	push eax
	drop
	loop .chain
.run:	call dword [eax]
next
;---------------------------------------------------------------------
hinstance rd 1
StdIn rd 1
StdOut rd 1
emit_buffer rd 1
written_buffer rd 1
rf_eax dd 0
rf_esi dd 0
tempX dd 0
;---------------------------------------------------------------------
include "retroforth.asm"

; windows-specific linkage stuff:
section '.idata' import data readable writeable

  library kernel,'KERNEL32.DLL'

  import kernel,\
	 GetModuleHandle,'GetModuleHandleA',\
	 ExitProcess,'ExitProcess',\
	 GetStdHandle, 'GetStdHandle',\
	 WriteFile, 'WriteFile',\
	 ReadFile, 'ReadFile',\
	 ReadConsole, 'ReadConsoleA',\
	 WriteConsole, 'WriteConsoleA',\
	 SetConsoleMode, 'SetConsoleMode',\
	 SetConsoleTitle, 'SetConsoleTitleA',\
	 CreateFile, 'CreateFileA',\
	 GetLastError,'GetLastError',\
	 CloseHandle,'CloseHandle',\
	 CreateConsoleScreenBuffer,'CreateConsoleScreenBuffer',\
	 SetConsoleActiveScreenBuffer,'SetConsoleActiveScreenBuffer',\
	 LoadLibrary,'LoadLibraryA',\
	 GetProcAddress,'GetProcAddress'

section '.edata' export data readable

  export 'WINDOWS.DLL',\
	 retroforth_attach,'retroforth_attach',\
	 eval,'retroforth_eval',\
	 retroforth_init, 'retroforth_init',\
	 retroforth_interpreter,'retroforth_interpreter'

section '.reloc' fixups data discardable
;---------------------------------------------------------------------
